// CWavIterator.cpp: implementation of the CWavIterator class.
//
//////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <conio.h>
#include "CWavIterator.h"
#include "SplitSound.h"
#include "options3.h"
#include "math.h"
#include <Forms.hpp>
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CWavIterator::CWavIterator()
{
	CommonConstruct();
}


CWavIterator::CWavIterator(char* pszFile)
{
	CommonConstruct();
	OpenFile(pszFile);

}



void CWavIterator::CommonConstruct()
{
	m_hWaveFile		= NULL;
	m_pRawSamples	= NULL;
	m_nBufferSize	= 4096;// read in up to 4024 samples of data at a time about 8k's worth)
	m_pRawSamples	= new short[m_nBufferSize+1];
	m_nTotalSamples = 0;

	// The buffers need to match for things to work....
	m_BufferMGR.m_nBufSize = m_nBufferSize;
	m_BufferMGR.Init();
}
CWavIterator::~CWavIterator()
{
	ResetContents();

	if(m_pRawSamples != NULL)
		delete [] m_pRawSamples;
	m_pRawSamples = NULL;
}
void CWavIterator::UseFilter(CZT_PcmBufferFilter& filter)
{
	m_BufferMGR.UseFilter(&filter);
}
void CWavIterator::ClearFilter()
{
	m_BufferMGR.UseFilter(NULL);
}
void CWavIterator::ResetContents()
{
	// close the file (if its opened)
	if(m_hWaveFile != NULL)
	{
		CloseHandle(m_hWaveFile);
		m_hWaveFile = NULL;
	}
	// clear internal buffer
	memset(m_pRawSamples,0,m_nBufferSize*2);
	m_nTotalSamples = 0;
	// Clear mgr..
	m_BufferMGR.ResetContents();
}



PCMPOS CWavIterator::OpenFile(char* pszFile)
{
	BOOL bSuccess = FALSE;


	ResetContents();

	// ok try and open the file before we do anything else....
	if(pszFile == NULL)
		return pcmpos_error;

	m_hWaveFile = CreateFile(pszFile,					
							GENERIC_READ,                 // open for reading 
							FILE_SHARE_READ,                            // do share
							NULL,                         // no security 
							OPEN_EXISTING,                // existing file only 
							FILE_ATTRIBUTE_NORMAL,        // normal file 
							NULL);                        // no attr. template

	// if it did not open we can not do anything
	if(m_hWaveFile == INVALID_HANDLE_VALUE)
	{
		m_hWaveFile = NULL;
		return pcmpos_error;
	}

	// first read the RIFF header...
	if(ReadRiffHeader())
	{
		// ok the riff was read ok...
		// now we can read in the format header
		if(ReadFormatHeader())
		{			
			// ok last but not least...
			if(ReadDataHeader())
			{
				// ok we are good to go...				
				bSuccess = TRUE;
			}
		}
	}

	if(!bSuccess)
	{
		ResetContents();
		return pcmpos_error;
	}

	// set the end pointers...
	m_lDataStart = SetFilePointer (m_hWaveFile,0, NULL, FILE_CURRENT);	
	m_lDataEnd   = SetFilePointer(m_hWaveFile,0,NULL,FILE_END);
	m_nTotalSamples = ((m_lDataEnd-m_lDataStart)/2.0f);


	// we are done..
	if(m_WaveHeader.nChannels==1)return GetHeadPosition();
        if(m_WaveHeader.nChannels==2)
        {
          //Application->MessageBox("stereo file", NULL, MB_OK);
          CWavFileSplitter	 mySplitter;
          ResetContents();
          if(mySplitter.OpenFile(pszFile))
          {
                 // save left channel into file...
                 mySplitter.SaveChannel(CWavFileSplitter::left,"C:\\SAP\\left_channel.wav");
                 // save right channel into file...
                 mySplitter.SaveChannel(CWavFileSplitter::right,"C:\\SAP\\right_channel.wav");
          }
          // resuse it
          mySplitter.CloseFile();
          if(option->channel->ItemIndex==0)pszFile="C:\\SAP\\left_channel.wav";
          else if(option->channel->ItemIndex==1)pszFile="C:\\SAP\\right_channel.wav";
          else  // select channel with stronger signal
          {
             pszFile="C:\\SAP\\left_channel.wav";
             OpenFile(pszFile);
             PCMPOS pcmPos = GetHeadPosition();
             int nFileTotal  =  GetTotalSamples();
             float *tmpData=new float[nFileTotal];
             ReadSamples(pcmPos, tmpData, nFileTotal);
             long int avgsound1=0;
             for(int i=0;i<nFileTotal;i++)
                   avgsound1+=fabs(tmpData[i]);
             delete[] tmpData;
             avgsound1/=nFileTotal;
             ResetContents();

             pszFile="C:\\SAP\\right_channel.wav";
             OpenFile(pszFile);
             pcmPos = GetHeadPosition();
             nFileTotal  =  GetTotalSamples();
             float *tmpData2=new float[nFileTotal];
             ReadSamples(pcmPos, tmpData2, nFileTotal);
             long int avgsound2=0;
             for(int i=0;i<nFileTotal;i++)
                   avgsound2+=fabs(tmpData2[i]);
             delete[] tmpData2;
             avgsound2/=nFileTotal;
             ResetContents();
             if(avgsound1>avgsound2)pszFile="C:\\SAP\\left_channel.wav";
             else pszFile="C:\\SAP\\right_channel.wav";

          }

          OpenFile(pszFile);
        }
        return(1);
}



inline BOOL	CWavIterator::ReadRiffHeader()
{
	DWORD dwRead = 0;
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.chkRiff,8,&dwRead,NULL))
		return FALSE;

	return m_WaveHeader.chkRiff.ckid == ztFOURCC_RIFF;
}


inline BOOL CWavIterator::ReadDataHeader()
{
	DWORD dwRead = 0;
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.chkData,8,&dwRead,NULL))
		return FALSE;

	return m_WaveHeader.chkData.ckid == ztFOURCC_DATA;
}


inline BOOL CWavIterator::ReadFormatHeader()
{
	DWORD dwRead = 0;
	// now we are looking for a fourcc WAVE 
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.fccWave,4,&dwRead,NULL))
		return FALSE;

	// make sure its what we expected..
	if(m_WaveHeader.fccWave != ztFOURCC_WAVE)
		return FALSE;

	// now we can read in the format header
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.chkFmt,8,&dwRead,NULL))
		return FALSE;	

	// we need to know the position so we can offset (at the end)
	DWORD dwEndOfFmt =SetFilePointer(m_hWaveFile,0,NULL,FILE_CURRENT);	
	dwEndOfFmt+= m_WaveHeader.chkFmt.dwSize;

	// make sure its what we expect
	if(m_WaveHeader.chkFmt.ckid != ztFOURCC_FMT)
		return FALSE;

	// read format tag...
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.wFormatTag,2,&dwRead,NULL))
		return FALSE;	

	// if the format is not 1 then we do not support it (1 = PCM)
	if(m_WaveHeader.wFormatTag != 1)
		return FALSE;

	// channels
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.nChannels,2,&dwRead,NULL))
		return FALSE;	


        // call wave splitter to split the sound, decide which channel to use, and make wave iterator
        // use that channels


	// nSamplesPerSec
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.nSamplesPerSec,4,&dwRead,NULL))
		return FALSE;	

	// nAvgBytesPerSec
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.nAvgBytesPerSec,4,&dwRead,NULL))
		return FALSE;	

	// Block Alignment
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.nBlockAlign,2,&dwRead,NULL))
		return FALSE;	

	// Bits Per Sample
	if(!ReadFile(m_hWaveFile,&m_WaveHeader.wBitsPerSample,2,&dwRead,NULL))
		return FALSE;	

	// ok we should be ok as long as we can move to the data chunk...
	return (SetFilePointer (m_hWaveFile,dwEndOfFmt, NULL, FILE_BEGIN) != INVALID_SET_FILE_POINTER);
}



PCMPOS CWavIterator::GetTailPosition()
{
	return FilePosToSamplePos(m_lDataEnd);
}



PCMPOS CWavIterator::GetHeadPosition()
{
	return FilePosToSamplePos(m_lDataStart);
}



int	CWavIterator::ReadSamples(PCMPOS& pos,float* pDest,int nCount)
{
	if(pos == pcmpos_end)
		return 0;// we can not read this..

	// They want data from pos thru pos + nCount...
	long lEnd = pos+nCount;
	int nFilled=0;// how many samples we fill this iteration

	// Clip to end if needed....
	if(lEnd > m_nTotalSamples)
	{
		lEnd = m_nTotalSamples;
		nCount = lEnd-pos;
	}

	// Lets see if we have any of that data stuffed away..
	int nRead = m_BufferMGR.ReadDataSegment(pos,lEnd,pDest);
	if(nRead != 0)
	{
		// Adjust the pos
		pos   += nRead;
		pDest += nRead;// also adjust the destination buffer..
		nFilled = nRead;

		// ok did we get all of it?
		if(nCount == nRead)
		{
			// if this was the last piece of data - then
			// mark their position as such
			if(SamplePosToFilePos(pos) == m_lDataEnd)
				pos=pcmpos_end;

			return nCount;// yup
		}
		// Adjust nCount
		nCount -=nRead;// and this
	}

	
	// ok so all of the data was not in the buffer...(maybe none of it was)
	// We need to read some more data into buffers and try again....
	// recursing into ourselves until the data request has been fulfilled, or
	// we run out of file to look at...
	if(FillNewBuffer(SamplePosToFilePos(pos))>0)
		return nFilled+ReadSamples(pos,pDest,nCount);

	return nFilled;
}
inline int CWavIterator::FillNewBuffer(LONG filePos)
{
	// Try and fill a new buffer..Return 0 if it did not happen...
	// Otherwise return the amount we filled...
	if(m_hWaveFile == NULL)
		return 0;// this is bad

	// move to our position
	DWORD dwTmp = SetFilePointer (m_hWaveFile,filePos, NULL, FILE_BEGIN);
	if(dwTmp == INVALID_SET_FILE_POINTER)
		return 0;//didnt happen

	// ok now we get to read..
	if(!ReadFile(m_hWaveFile,m_pRawSamples,m_nBufferSize*2,&dwTmp,NULL))
		return 0;// uh oh

	// Ok we read some data...
	int nRead = int(dwTmp/2.0f);

	if(nRead <=0)
		return 0;// jic

	// Get a buffer and Copy our data into it...
	CZT_PcmBuffer* pCurBuffer = m_BufferMGR.GetBufferForWriting();
	pCurBuffer->SetData(m_pRawSamples,FilePosToSamplePos(filePos),nRead);

	return nRead;
}
inline LONG CWavIterator::SamplePosToFilePos(PCMPOS pos)
{
	return m_lDataStart+(pos*2);// *2 because in wave file samples are 2bytes...
}
inline PCMPOS CWavIterator::FilePosToSamplePos(LONG pos)
{
	// File position to sample position..
	// fileposition-m_lDataStart
	return LONG((pos-m_lDataStart)/2.0f);
}

